Option Explicit On 
Option Strict On
Imports System.Web.Services
Imports System.Web.Services.Protocols
Imports System.Data.SqlClient
Imports System.Xml
Imports System.Xml.Schema
Imports System.IO

Public Class Authenticate
   Inherits SoapHeader
   Public ServerName As String
   Public UserName As String
   Public Password As String
End Class

<WebService(Namespace:="http://oakleaf.ws/services/crosstabs/", _
   Description:="<p><b>ReportsWS</b> is an XML Web service that provides methods to return " + _
   "a variety of crosstab(ular) financial reports from the OakLeaf Consumer " + _
   "Electronics Orders, Invoices, and Products tables of the OCE_Orders SQL Server 2000 sample database. The tables currently store data for about 15,000 orders and 45,000 line items. The service uses " + _
   "XML documents for SOAP request and response messages. Only the <font size='2', face='lucida console, courier new'>TimeSeriesReport</font> and <font size='2', face='lucida console, courier new'>TimeSeriesValidated</font> methods are implemented at this time.</p>" + _
   "<p>The <a href='http://www.oakleaf.ws/CTReports/CTReport.aspx'>CTReports.aspx</a> ASP.NET client application lets you specify report parameters, such as starting date, interval (quarterly, monthly, weekly or daily), the number of columns, row and column totals, and currency formatting.</p>" + _
   "<p><b>Background: </b>ReportsWS is one of a series of data-driven XML Web service examples from <i>Visual Basic .NET XML Web Services Developer's Guide</i> (McGraw-Hill/Osborne, ISBN 0-07-222369-3, Spring 2002). Click <a href='http://www.oakleaf.ws/CTReports/AboutCTR.aspx'>here</a> for a detailed description of the ReportsWS XML Web service and its source code.</p>" & _
   "<p>Click <a href='http://www.oakleaf.ws/CROrdersCT/OrdersReport.aspx'>here</a> for an example of a Crystal Reports for Visual Studio .NET crosstab report and chart generated from a Web service based on the same data.</p>" + _
   "<p><center>XML Web service, documentation, and source code Copyright &#169; 2002 by OakLeaf Systems and Roger Jennings.<br />All rights reserved.</center></p>")> _
Public Class ReportsWS
   Inherits System.Web.Services.WebService
   Private strXML As String
   Private strCrLf As String = Chr(13) + Chr(10)
   Private strTitle As String = "2001 Quarterly Orders by State"
   Private blnNullToZero As Boolean = True
   Private blnRowTotals As Boolean = True
   Private blnColTotals As Boolean = True

   'Validation schema cache (copied from validating client code)
   Private xscCrosstabs As XmlSchemaCollection
   Private blnValid As Boolean

   'Variables for creating T-SQL queries
   Private strStartDate As String = "1/1/2001"
   Private datStartDate As DateTime = #1/1/2001#
   Private strStartTime As String = "00:00:00 AM"
   Private strInterval As String = "Quarter"
   Private strTitleInt As String = "Quarterly"
   Private strDataCols As String = "4"
   Private strQuery As String
   Private blnStates As Boolean = True

   Public hdrAuthent As Authenticate

#Region " Web Services Designer Generated Code "

   Public Sub New()
      MyBase.New()

      'This call is required by the Web Services Designer.
      InitializeComponent()

      'Add your own initialization code after the InitializeComponent() call
      Trace.Write("Start XSD schema caching")
      xscCrosstabs = New XmlSchemaCollection()
      With xscCrosstabs
         'Add the schema file to the cache
         .Add("http://oakleaf.ws/services/crosstabs/request", "http://www.oakleaf.ws/ReportsWS/CTReqTyped.xsd")
      End With
      Trace.Write("End XSD schema caching")
   End Sub

    'Required by the Web Services Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Web Services Designer
    'It can be modified using the Web Services Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container()
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        'CODEGEN: This procedure is required by the Web Services Designer
        'Do not modify it using the code editor.
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

#End Region

   '****************************************
   'Validating Version (TimeSeriesValidated)
   'Modified clone of TimeSeriesReport
   'Validates against CTReqTyped.xsd schema
   '****************************************
   <WebMethod(Description:="<br /><p>The <font size='2', face='lucida console, courier new'>TimeSeriesValidated</font> method is nearly identical to the <font size='2', face='lucida console, courier new'>TimeSeriesReport</font> method. " + _
      "This alternative method uses the <font size='2', face='lucida console, courier new'>XmlValidatingReader</font> object to validate the structure and datatypes of a strongly-typed request message against an XSD schema document generated by Visual Studio .NET's XML Designer from the XML request document shown below. " + _
      "The schema specifies appropriate XSD datatypes for each element: <font size='2', face='lucida console, courier new'>xsd:string</font>, <font size='2', face='lucida console, courier new'>xsd:date</font>, <font size='2', face='lucida console, courier new'>xsd:time</font>, <font size='2', face='lucida console, courier new'>xsd:nonNegativeInteger</font>, or <font size='2', face='lucida console, courier new'>xsd:boolean</font>. </p>" + _
      "<p>Following is an example of a typical SOAP request document. Element value changes required to conform to ISO 8601 CCYY-MM-DD date and HH:MM:SS time formats, and Boolean values appear in boldface.</p>" + _
      "<pre><font size='2', face='lucida console, courier new'>" + _
      "&lt;crosstab xmlns='http://oakleaf.ws/services/crosstabs/request' &gt;<br />" + _
      "   &lt;title&gt;2001 Quarterly Orders by State&lt;/title&gt;<br />" + _
      "   &lt;timeseries&gt;<br />" + _
      "      &lt;datatable&gt;Orders&lt;/datatable&gt;<br />" + _
      "      &lt;rowsby&gt;State&lt;/rowsby&gt;<br />" + _
      "      &lt;startdate&gt;<b>2001-01-01</b>&lt;/startdate&gt;<br />" + _
      "      &lt;starttime&gt;<b>00:00:00</b>&lt;/starttime&gt;<br />" + _
      "      &lt;interval&gt;Quarter&lt;/interval&gt;<br />" + _
      "      &lt;datacolumns&gt;4&lt;/datacolumns&gt;<br />" + _
      "      &lt;count&gt;<b>false</b>&lt;/count&gt;<br />" + _
      "      &lt;avgvalue&gt;<b>false</b>&lt;/avgvalue&gt;<br />" + _
      "      &lt;rowtotals&gt;<b>true</b>&lt;/rowtotals&gt;<br />" + _
      "      &lt;coltotals&gt;<b>true</b>&lt;/coltotals&gt;<br />" + _
      "      &lt;nulltozero&gt;<b>true</b>&lt;/nulltozero&gt;<br />" + _
      "      &lt;format&gt;XML&lt;/format&gt;<br />" + _
      "   &lt;/timeseries&gt;<br />" + _
      "&lt;/crosstab&gt;<br />" + _
      "</font></pre>" + _
      "<p><b>Schema:</b> Click <a href='http://www.oakleaf.ws/ReportsWS/CTReqTyped.xsd'>here</a> to open the XML Schema (XSD) validation file for the request message.</p>" + _
      "<p>Chapter 10, ""Applying Advanced XML Web Service Techniques,"" of <i>Visual Basic .NET XML Web Services Developer's Guide</i> describes the ASP.NET client for this method, which also validates the request document, </p>" _
   , CacheDuration:=60), SoapHeader("hdrAuthent", Required:=False)> _
   Public Function TimeSeriesValidated(ByVal RequestDoc As String) As String
      'Add LogMessages("\VBWS10\ReportsWS\TimeSeriesValidated.log", 0) after 
      'SoapHeader(... above to to enable logging
      'Verify the request document prior to reading it
      Dim srdRequest As StringReader = Nothing
      Dim xtrRequest As XmlTextReader = Nothing
      Dim strTemp As String
      Dim strSQL As String
      Dim strResponse As String

      Try
         'Copied from client code and modified for this method
         Trace.Write("Start validate request message")
         srdRequest = New StringReader(RequestDoc)
         xtrRequest = New XmlTextReader(srdRequest)
         Dim xvrRequest As New XmlValidatingReader(xtrRequest)
         blnValid = True
         With xvrRequest
            .Schemas.Add(xscCrosstabs)
            AddHandler .ValidationEventHandler, AddressOf ValidationError
            While .Read
               'Process the document
            End While
            .Close()
         End With
         Trace.Write("End validate request message")
         If Not blnValid Then
            'Return nothing
            Return ""
            Exit Function
         End If
         Trace.Write("Begin read request document")
         srdRequest = New StringReader(RequestDoc)
         xtrRequest = New XmlTextReader(srdRequest)
         With xtrRequest
            '(strTemp reads elements not used at this point)
            .MoveToContent()  'Root
            .Read()           'Skip <timeseries>
            strTitle = .ReadElementString("title")
            .Read()           'Skip
            .Read()           'Skip
            strTemp = .ReadElementString("datatable")
            If .ReadElementString("rowsby") = "State" Then
               blnStates = True
            Else
               blnStates = False
            End If
            strStartDate = .ReadElementString("startdate")
            strStartTime = .ReadElementString("starttime")
            strInterval = .ReadElementString("interval")
            strDataCols = .ReadElementString("datacolumns")
            strTemp = .ReadElementString("count")
            strTemp = .ReadElementString("avgvalue")
            If .ReadElementString("rowtotals") = "true" Then
               blnRowTotals = True
            Else
               blnRowTotals = False
            End If
            If .ReadElementString("coltotals") = "true" Then
               blnColTotals = True
            Else
               blnColTotals = False
            End If
            If .ReadElementString("nulltozero") = "true" Then
               blnNullToZero = True
            Else
               blnNullToZero = False
            End If
         End With
      Catch
         'No error processing at this point
      Finally
         srdRequest.Close()
         xtrRequest.Close()
      End Try
      'Generate the T-SQL statement
      strSQL = WriteOrdersQuery()
      'Execute the query and return the report document
      strResponse = GenerateReport(strSQL)
      Return strResponse
   End Function

   Private Sub ValidationError(ByVal sender As Object, ByVal args As ValidationEventArgs)
      'Common callback function for intercepting request and response validation errors
      blnValid = False
   End Sub

   '****************************************
   'Non-validating Version (TimeSeriesReport)
   'Original method
   '****************************************
   <WebMethod(Description:="<br /><p>The <font size='2', face='lucida console, courier new'>TimeSeriesReport</font> method accepts an element-centric XML document that specifies " + _
      "the data source and layout of a time-series crosstab report for orders received by state or product category. The method returns an element-centric " + _
      "XML document with headers and formatted row and column values. You can't use the test form to execute the ReportsWS methods because they require SOAP header element values for SQL Server authentication.</p>" + _
      "<p>Following is an example of a typical SOAP request document:</p>" + _
      "<pre><font size='2', face='lucida console, courier new'>" + _
      "&lt;crosstab xmlns='http://oakleaf.ws/services/crosstabs/request' &gt;<br />" + _
      "   &lt;title&gt;2001 Quarterly Orders by State&lt;/title&gt;<br />" + _
      "   &lt;timeseries&gt;<br />" + _
      "      &lt;datatable&gt;Orders&lt;/datatable&gt;<br />" + _
      "      &lt;rowsby&gt;State&lt;/rowsby&gt;<br />" + _
      "      &lt;startdate&gt;1/1/2001&lt;/startdate&gt;<br />" + _
      "      &lt;starttime&gt;00:00:00AM&lt;/starttime&gt;<br />" + _
      "      &lt;interval&gt;Quarter&lt;/interval&gt;<br />" + _
      "      &lt;datacolumns&gt;4&lt;/datacolumns&gt;<br />" + _
      "      &lt;count&gt;No&lt;/count&gt;<br />" + _
      "      &lt;avgvalue&gt;No&lt;/avgvalue&gt;<br />" + _
      "      &lt;rowtotals&gt;Yes&lt;/rowtotals&gt;<br />" + _
      "      &lt;coltotals&gt;Yes&lt;/coltotals&gt;<br />" + _
      "      &lt;nulltozero&gt;Yes&lt;/nulltozero&gt;<br />" + _
      "      &lt;format&gt;XML&lt;/format&gt;<br />" + _
      "   &lt;/timeseries&gt;<br />" + _
      "&lt;/crosstab&gt;<br />" + _
      "</font></pre>" + _
      "<p><b>Schema:</b> Click <a href='http://www.oakleaf.ws/CTReports/CTRequest.xsd'>here</a> to open the XML Schema (XSD) file for the request message.</p>" + _
      "<p>The preceding request document returns the following SOAP response document:</p>" + _
      "<pre><font size='2', face='lucida console, courier new'>" + _
      "&lt;crosstab xmlns='http://oakleaf.ws/services/crosstabs/response' &gt;<br />" + _
      "&lt;title&gt;2001 Quarterly Orders by State&lt;/title&gt;<br />" + _
      "&lt;header&gt;<br />" + _
      "   &lt;hdrcols&gt;<br />" + _
      "      &lt;hdrcol id='hdr_col_State' cell='R1C1'&gt;State&lt;/hdrcol&gt;<br />" + _
      "      &lt;hdrcol id='hdr_col_2001Q1' cell='R1C2'&gt;2001Q1&lt;/hdrcol&gt;<br />" + _
      "      &lt;hdrcol id='hdr_col_2001Q2' cell='R1C3'&gt;2001Q2&lt;/hdrcol&gt;<br />" + _
      "      &lt;hdrcol id='hdr_col_2001Q3' cell='R1C4'&gt;2001Q3&lt;/hdrcol&gt;<br />" + _
      "      &lt;hdrcol id='hdr_col_2001Q4' cell='R1C5'&gt;2001Q4&lt;/hdrcol&gt;<br />" + _
      "      &lt;hdrcol id='hdr_col_Totals' cell='R1C6'&gt;Totals&lt;/hdrcol&gt;<br />" + _
      "   &lt;/hdrcols&gt;<br />" + _
      "&lt;/header&gt;<br />" + _
      "&lt;data&gt;<br />" + _
      "   &lt;row&gt;<br />" + _
      "      &lt;col id='hdr_row_CO' cell='R2C1'&gt;CO&lt;/col&gt;<br />" + _
      "      &lt;col id='st_CO_2001Q1' cell='R2C2'&gt;$166,419&lt;/col&gt;<br />" + _
      "      &lt;col id='st_CO_2001Q2' cell='R2C3'&gt;$343,470&lt;/col&gt;<br />" + _
      "      &lt;col id='st_CO_2001Q3' cell='R2C4'&gt;$584,724&lt;/col&gt;<br />" + _
      "      &lt;col id='st_CO_2001Q4' cell='R2C5'&gt;$802,211&lt;/col&gt;<br />" + _
      "      &lt;col id='total_row2' cell='R2C6'&gt;$1,896,824&lt;/col&gt;<br />" + _
      "   &lt;/row&gt;<br />" + _
      "   ...<br />" + _
      "   &lt;row&gt;<br />" + _
      "      &lt;col id='hdr_row_WY' cell='R13C1'&gt;WY&lt;/col&gt;<br />" + _
      "      &lt;col id='st_WY_2001Q1' cell='R13C2'&gt;11,449&lt;/col&gt;<br />" + _
      "      &lt;col id='st_WY_2001Q2' cell='R13C3'&gt;29,451&lt;/col&gt;<br />" + _
      "      &lt;col id='st_WY_2001Q3' cell='R13C4'&gt;88,883&lt;/col&gt;<br />" + _
      "      &lt;col id='st_WY_2001Q4' cell='R13C5'&gt;99,493&lt;/col&gt;<br />" + _
      "      &lt;col id='total_row13' cell='R13C6'&gt;$229,276&lt;/col&gt;<br />" + _
      "   &lt;/row&gt;<br />" + _
      "   &lt;row&gt;<br />" + _
      "      &lt;col id='total_col1' cell='R14C1'&gt;Totals&lt;/col&gt;<br />" + _
      "      &lt;col id='total_col2' cell='R14C2'&gt;$1,553,373&lt;/col&gt;<br />" + _
      "      &lt;col id='total_col3' cell='R14C3'&gt;$5,178,609&lt;/col&gt;<br />" + _
      "      &lt;col id='total_col4' cell='R14C4'&gt;$10,902,432&lt;/col&gt;<br />" + _
      "      &lt;col id='total_col5' cell='R14C5'&gt;$18,813,088&lt;/col&gt;<br />" + _
      "      &lt;col id='total_col6' cell='R14C6'&gt;$37,396,900&lt;/col&gt;<br />" + _
      "   &lt;/row&gt;<br />" + _
      "&lt;/data&gt;<br />" + _
      "&lt;/crosstab&gt;</font></pre>" + _
      "<p><b>Note:</b> Rows 3 through 13 for Iowa through Texas have been deleted from the preceding example for brevity.</p>" + _
      "<p><b>Schema:</b> Click <a href='http://www.oakleaf.ws/CTReports/CTResponse.xsd'>here</a> to open the XML Schema (XSD) file for the response message.</p>" + _
      "<p>If you want to write a consuming application, use the following SOAP hearder values:<br /><ul>" + _
      "<li>ServerName = <b>OAKLEAF-MS7</b></li><li>UserName = <b>CustUser</b></li><li>Password = <b>Cust#123</b></li></ul>" _
      , CacheDuration:=60), SoapHeader("hdrAuthent", Required:=False), LogMessages("\VBWS10\ReportsWS\TimeSeriesReport.log", 0)> _
   Public Function TimeSeriesReport(ByVal RequestDoc As String) As String
      'Add LogMessages("\VBWS10\ReportsWS\TimeSeriesReport.log", 0) after 
      'SoapHeader(... above to to enable logging

      'Read the request document and verify its elements
      Dim srdRequest As StringReader = Nothing
      Dim xtrRequest As XmlTextReader = Nothing
      Dim strTemp As String
      Dim strSQL As String
      Dim strResponse As String

      Try
         Trace.Write("Begin read request document")
         srdRequest = New StringReader(RequestDoc)
         xtrRequest = New XmlTextReader(srdRequest)
         With xtrRequest
            '(strTemp reads elements not used at this point)
            .MoveToContent()  'Root
            .Read()           'Skip <timeseries>
            strTitle = .ReadElementString("title")
            .Read()           'Skip
            .Read()           'Skip
            strTemp = .ReadElementString("datatable")
            If .ReadElementString("rowsby") = "State" Then
               blnStates = True
            Else
               blnStates = False
            End If
            strStartDate = .ReadElementString("startdate")
            strStartTime = .ReadElementString("starttime")
            strInterval = .ReadElementString("interval")
            strDataCols = .ReadElementString("datacolumns")
            strTemp = .ReadElementString("count")
            strTemp = .ReadElementString("avgvalue")
            strTemp = .ReadElementString("rowtotals")
            If strTemp = "Yes" Or strTemp = "true" Then
               blnRowTotals = True
            Else
               blnRowTotals = False
            End If
            strTemp = .ReadElementString("coltotals")
            If strTemp = "Yes" Or strTemp = "true" Then
               blnColTotals = True
            Else
               blnColTotals = False
            End If
            strTemp = .ReadElementString("nulltozero")
            If strTemp = "Yes" Or strTemp = "true" Then
               blnNullToZero = True
            Else
               blnNullToZero = False
            End If
         End With
      Catch
         'No error processing at this point
      Finally
         srdRequest.Close()
         xtrRequest.Close()
      End Try
      'Generate the T-SQL statement
      strSQL = WriteOrdersQuery()
      'Execute the query and return the report document
      strResponse = GenerateReport(strSQL)
      Return strResponse
   End Function

   '********************************************
   'T-SQL Query and Report Generation Procedures
   '********************************************

   Private Function WriteOrdersQuery() As String
      'Generate the T-SQL query from request document values
      Dim strSelect As String
      Dim strField As String
      Dim strHeader As String
      Dim strPeriod As String
      Dim strYear As String
      Dim datLastDate As DateTime
      Dim strFrom As String
      Dim strWhere As String
      Dim strGroup As String
      Dim strOrder As String
      Dim strSQL As String

      Trace.Write("Begin generate T-SQL query")
      datStartDate = CDate(strStartDate)
      datLastDate = datStartDate
      strFrom = "FROM Customers " + _
         "INNER JOIN Orders ON Customers.CustID = Orders.CustID " + _
         "INNER JOIN LineItems ON Orders.OrderID = LineItems.OrderID "

      strSelect = "SELECT TOP 100 PERCENT "
      strYear = "DATEPART(year, OrderDate)"
      strWhere = "WHERE (OrderDate BETWEEN '" + strStartDate + " " + strStartTime + "' AND '"
      strGroup = "GROUP BY "
      strOrder = "ORDER BY "
      If blnStates Then
         strField = "State"
         strSelect += strField + ", "
      Else
         strField = "SUBSTRING(SKU, 1, 4)"
         strSelect += strField + " AS Category, "
      End If
      strGroup += strField + ", "
      strOrder += strField + ", "

      Select Case strInterval
         Case "Quarter"
            strHeader = "DATENAME(year, OrderDate) + 'Q' + DATENAME(quarter, OrderDate)"
            strPeriod = "DATEPART(quarter, OrderDate)"
            datLastDate = datLastDate.AddMonths(3 * CInt(strDataCols))
         Case "Month"
            strHeader = "SUBSTRING(DATENAME(month, OrderDate), 1, 3) + ' ' + DATENAME(year, OrderDate)"
            strPeriod = "DATEPART(month, OrderDate)"
            datLastDate = datLastDate.AddMonths(CInt(strDataCols))
         Case "Week"
            strHeader = "DATENAME(year, OrderDate) + 'W' + DATENAME(week, OrderDate)"
            strPeriod = "DATEPART(week, OrderDate)"
            datLastDate = datLastDate.AddDays(7 * CInt(strDataCols))
         Case "Day"
            strHeader = "CAST(DATEPART(mm, OrderDate) AS varchar) + '/' + CAST(DATEPART(dd, OrderDate) AS varchar) + '/' + CAST(DATEPART(yy, OrderDate) AS varchar)"
            strPeriod = "DATEPART(dy, OrderDate)"
            datLastDate = datLastDate.AddDays(CInt(strDataCols))
      End Select
      datLastDate = datLastDate.AddSeconds(-1)
      strGroup += strPeriod + ", " + strYear + ", " + strHeader + " "
      strOrder += strPeriod + ", " + strYear + ", " + strHeader + " "
      strPeriod += " AS Period, " + strYear + " AS Year "
      strSelect += strHeader + " AS Header, COUNT(Orders.OrderID) AS Orders, ROUND(SUM(Quantity * UnitPrice), 0) AS Amount, " + strPeriod
      strWhere += Format(datLastDate, "M/d/yyyy hh:mm:ss tt") + "') AND LineItems.Deleted = 0 "

      strSQL = strSelect + strCrLf + strFrom + strCrLf + strWhere + strCrLf + strGroup + strCrLf + strOrder
      Return strSQL
   End Function

   Private Function GenerateReport(ByVal strSQL As String) As String
      'Generate and populate array, and create basic XML response document
      'SOAP headers supply ServerName, UserName, and Password values
      Dim astrCT(,) As String
      Dim intCol As Int32
      Dim intRow As Int32
      Dim intTotal As Int32
      Dim datPeriod As DateTime
      Dim cnnCT As New SqlConnection()
      Dim cmmCT As SqlCommand
      Dim sdrCT As SqlDataReader
      Dim strTemp As String
      Dim strAttrib As String
      Dim strXML As String 'Response document

      Trace.Write("Begin create array")
      datPeriod = datStartDate
      If blnStates Then
         ReDim astrCT(13, CInt(strDataCols) + 1)
         astrCT(0, 0) = "State"
         astrCT(1, 0) = "CO"
         astrCT(2, 0) = "IA"
         astrCT(3, 0) = "KS"
         astrCT(4, 0) = "MO"
         astrCT(5, 0) = "MT"
         astrCT(6, 0) = "ND"
         astrCT(7, 0) = "NE"
         astrCT(8, 0) = "NM"
         astrCT(9, 0) = "OK"
         astrCT(10, 0) = "SD"
         astrCT(11, 0) = "TX"
         astrCT(12, 0) = "WY"
         astrCT(13, 0) = "Totals"
      Else
         ReDim astrCT(13, CInt(strDataCols) + 1)
         astrCT(0, 0) = "Category"
         astrCT(1, 0) = "AMPL"
         astrCT(2, 0) = "CAMC"
         astrCT(3, 0) = "CASD"
         astrCT(4, 0) = "CDPL"
         astrCT(5, 0) = "DVDP"
         astrCT(6, 0) = "RCVA"
         astrCT(7, 0) = "RCVH"
         astrCT(8, 0) = "SPKA"
         astrCT(9, 0) = "SPKB"
         astrCT(10, 0) = "SPKF"
         astrCT(11, 0) = "SPKS"
         astrCT(12, 0) = "TVRC"
         astrCT(13, 0) = "Totals"
      End If

      'Generate the time-series column headers
      For intCol = 1 To CInt(strDataCols)
         Select Case strInterval
            Case "Quarter"
               datPeriod = datStartDate.AddMonths(3 * (intCol - 1))
               astrCT(0, intCol) = CStr(datPeriod.Year) + "Q" + CStr((datPeriod.Month \ 3) + 1)
            Case "Month"
               datPeriod = datStartDate.AddMonths(intCol - 1)
               astrCT(0, intCol) = Format(datPeriod, "MMM") + " " + CStr(datPeriod.Year)
            Case "Week"
               datPeriod = datStartDate.AddDays(7 * (intCol - 1))
               If datStartDate = #12/31/2000# And intCol = 1 Then
                  'Special case for 2001, which starts on Monday
                  astrCT(0, intCol) = "2001W1"
               Else
                  astrCT(0, intCol) = CStr(datPeriod.Year) + "W" + CStr((datPeriod.DayOfYear \ 7) + 1)
               End If
            Case "Day"
               datPeriod = datStartDate.AddDays(intCol - 1)
               astrCT(0, intCol) = Format(datPeriod, "M/d/yyyy")
         End Select
      Next intCol
      astrCT(0, CInt(strDataCols) + 1) = "Totals"

      'Fill array with 0s
      For intRow = 1 To UBound(astrCT, 1)
         For intCol = 1 To UBound(astrCT, 2)
            astrCT(intRow, intCol) = "0"
         Next intCol
      Next intRow

      'Connect and create a SqlDataReader
      Trace.Write("Begin connect to server")
      With cnnCT
         If hdrAuthent Is Nothing Then
            .ConnectionString = "Data Source=(local); " + "Initial Catalog=OCE_Cust;UID=CustUser;PWD=Cust#123"
         Else
            .ConnectionString = "Data Source=" + hdrAuthent.ServerName + ";Initial Catalog=OCE_Cust;UID=" + hdrAuthent.UserName + ";PWD=" + hdrAuthent.Password
         End If
         .Open()
         cmmCT = .CreateCommand
      End With
      With cmmCT
         .CommandType = CommandType.Text
         .CommandText = strSQL
         sdrCT = .ExecuteReader
      End With
      Trace.Write("Begin fill array with data")
      With sdrCT
         While .Read
            'Iterate the rowset and plug values into the proper cell
            For intRow = 1 To UBound(astrCT, 1)
               'Find the row
               If sdrCT.GetString(0) = astrCT(intRow, 0) Then
                  Exit For
               End If
            Next intRow
            For intCol = 1 To UBound(astrCT, 2)
               'Find the column
               If sdrCT.GetString(1) = astrCT(0, intCol) Then
                  Exit For
               End If
            Next intCol
            If intRow < UBound(astrCT, 1) And intCol < UBound(astrCT, 2) Then
               'Found the cell; add the value without decimals
               astrCT(intRow, intCol) = CStr(CInt(sdrCT.GetDecimal(3)))
            Else
               'Error below for test
               strTemp = sdrCT.GetString(0) + ", " + sdrCT.GetString(1) + ", " + CStr(sdrCT.GetDecimal(3))
            End If
         End While
      End With
      sdrCT.Close()
      cnnCT.Close()
      'Get row totals
      Trace.Write("Begin row/column totals")
      For intRow = 1 To UBound(astrCT, 1) - 1
         intTotal = 0
         For intCol = 1 To UBound(astrCT, 2) - 1
            intTotal += CInt(astrCT(intRow, intCol))
         Next intCol
         astrCT(intRow, UBound(astrCT, 2)) = CStr(intTotal)
      Next intRow
      'Get column totals (including crossfoot total)
      For intCol = 1 To UBound(astrCT, 2)
         intTotal = 0
         For intRow = 1 To UBound(astrCT, 1) - 1
            intTotal += CInt(astrCT(intRow, intCol))
         Next intRow
         astrCT(UBound(astrCT, 1), intCol) = CStr(intTotal)
      Next intCol
      'Format rows and columns
      For intRow = 1 To UBound(astrCT, 1)
         For intCol = 1 To UBound(astrCT, 2)
            If intRow = 1 Or intRow = UBound(astrCT, 1) Or intCol = UBound(astrCT, 2) Then
               astrCT(intRow, intCol) = Format(CInt(astrCT(intRow, intCol)), "$#,##0")
            Else
               astrCT(intRow, intCol) = Format(CInt(astrCT(intRow, intCol)), "#,##0")
            End If
         Next intCol
      Next intRow

      'Generate the response document
      Trace.Write("Begin generate response")
      strXML = "<crosstab xmlns='http://oakleaf.ws/services/crosstabs/response' >" + strCrLf
      For intRow = 0 To UBound(astrCT, 1)
         If intRow = UBound(astrCT, 1) And Not blnColTotals Then
            Exit For
         End If
         If intRow = 0 Then
            'Title
            strXML += "   <title>" + strTitle + "</title>" + strCrLf
            'Header
            'strXML += "   <header>" + strCrLf + Space(6) + "<row>" + strCrLf
            strXML += "   <header>" + strCrLf + Space(6) + "<hdrcols>" + strCrLf
            For intCol = 0 To UBound(astrCT, 2)
               If intCol = UBound(astrCT, 2) And Not blnRowTotals Then
                  Exit For
               End If
               strAttrib = "id='hdr_col_" + astrCT(0, intCol) + "' cell='R1C" + CStr(intCol + 1) + "'"
               'strXML += Space(9) + "<col " + strAttrib + ">" + astrCT(intRow, intCol) + "</col>" + strCrLf
               strXML += Space(9) + "<hdrcol " + strAttrib + ">" + astrCT(intRow, intCol) + "</hdrcol>" + strCrLf
            Next intCol
            'strXML += Space(6) + "</row>" + strCrLf + "   </header>" + strCrLf
            strXML += Space(6) + "</hdrcols>" + strCrLf + "   </header>" + strCrLf
            strXML += "   <data>" + strCrLf
         Else
            'Data rows
            strXML += Space(6) + "<row>" + strCrLf
            For intCol = 0 To UBound(astrCT, 2)
               If intCol = UBound(astrCT, 2) And Not blnRowTotals Then
                  Exit For
               End If
               If intRow = UBound(astrCT, 1) Then
                  strAttrib = "id='total_col" + CStr(intCol + 1) + "' cell='R" + CStr(intRow + 1) + "C" + CStr(intCol + 1) + "'"
               ElseIf intCol = UBound(astrCT, 2) Then
                  strAttrib = "id='total_row" + CStr(intRow + 1) + "' cell='R" + CStr(intRow + 1) + "C" + CStr(intCol + 1) + "'"
               Else
                  If intCol > 0 Then
                     If blnStates Then
                        strAttrib = "id='st_"
                     Else
                        strAttrib = "id='cat_"
                     End If
                     strAttrib += astrCT(intRow, 0) + "_" + astrCT(0, intCol)
                  Else
                     strAttrib = "id='hdr_row_" + astrCT(intRow, 0)
                  End If
                  strAttrib += "' cell='R" + CStr(intRow + 1) + "C" + CStr(intCol + 1) + "'"
               End If
               If (astrCT(intRow, intCol) = "0" Or astrCT(intRow, intCol) = "$0") And Not blnNullToZero Then
                  'Empty element for zero values
                  strXML += Space(9) + "<col " + strAttrib + "></col>" + strCrLf
               Else
                  strXML += Space(9) + "<col " + strAttrib + ">" + astrCT(intRow, intCol) + "</col>" + strCrLf
               End If
            Next intCol
            strXML += Space(6) + "</row>" + strCrLf
         End If
      Next intRow
      'Close the tags
      strXML += "   </data>" + strCrLf
      strXML += "</crosstab>"
      Trace.Write("End generate response")
      Return strXML
   End Function
End Class

'**************************************************************************
'LogMessages SOAP Extension derived from the TraceExtension sample
'in the "Altering the SOAP Message Using SOAP Extensions" online help topic
'**************************************************************************

'Define an attribute to specify the log file name and location,
'and the extension's priority
<AttributeUsage(AttributeTargets.Method)> Public Class LogMessagesAttribute
   Inherits SoapExtensionAttribute

   'Supply default values
   Private m_intPriority As Integer = 0
   Private m_strLogFile As String = "\VBWS10\ReportsWS\Report.log"

   'Override the default values
   Public Sub New(ByVal strLogFile As String, ByVal intPriority As Integer)
      m_strLogFile = strLogFile
      m_intPriority = intPriority
   End Sub

   Public ReadOnly Property strLogFile() As String
      Get
         strLogFile = m_strLogFile
      End Get
   End Property

   Public Overrides Property Priority() As Integer
      Get
         Priority = m_intPriority
      End Get
      Set(ByVal Value As Integer)
         m_intPriority = Value
      End Set
   End Property

   Public Overrides ReadOnly Property ExtensionType() As Type
      Get
         Return GetType(LogMessages)
      End Get
   End Property
End Class

'Applying the LogMessages("\Path\Name.log", Priority) attribute 
'to a WebMethod instantiates the following class derived from SoapExtension
Public Class LogMessages
   Inherits SoapExtension

   Private strLogFile As String
   Private stmSoap As Stream
   Private stmTemp As Stream
   Private lngTicks As Long

   Public Overloads Overrides Function GetInitializer(ByVal serviceType As Type) As Object
      'Get the service type
      Return Nothing
   End Function

   Public Overloads Overrides Function GetInitializer(ByVal methodInfo As LogicalMethodInfo, ByVal attribute As SoapExtensionAttribute) As Object
      'Get the initializer for the strLogFile attribute
      Dim atrLog As LogMessagesAttribute = CType(attribute, LogMessagesAttribute)
      Return atrLog.strLogFile
   End Function

   Public Overrides Sub Initialize(ByVal initializer As Object)
      'Get the strLogFile value from the initializer
      strLogFile = initializer.ToString
   End Sub

   Public Overrides Function ChainStream(ByVal stream As Stream) As Stream
      'Replace the default soap message streams with the stmSoap and stmTemp
      stmSoap = stream
      stmTemp = New MemoryStream()
      Return stmTemp
   End Function

   Public Overrides Sub ProcessMessage(ByVal message As SoapMessage)
      'Log the serialized messages at the appropriate stages
      Select Case message.Stage
         Case SoapMessageStage.BeforeDeserialize
            'Get time and write the request message
            'Ticks are 100 ns increments
            lngTicks = DateTime.Now.Ticks
            Call WriteRequest()
         Case SoapMessageStage.AfterDeserialize
            'Do nothing
         Case SoapMessageStage.BeforeSerialize
            'Do nothing
         Case SoapMessageStage.AfterSerialize
            'Get elapsed time and write the response message
            lngTicks = DateTime.Now.Ticks - lngTicks
            Call WriteResponse(lngTicks)
      End Select
   End Sub

   Sub WriteRequest()
      'Append the request message to the log and return the stream to the service
      CopyStream(stmSoap, stmTemp)
      Dim fsRequest As New FileStream(strLogFile, FileMode.Append, FileAccess.Write)
      Dim swRequest As New StreamWriter(fsRequest)
      swRequest.WriteLine("<!-- SOAP Request at " + DateTime.Now.ToString + " -->")
      swRequest.Flush()
      stmTemp.Position = 0
      CopyStream(stmTemp, fsRequest)
      fsRequest.Close()
      stmTemp.Position = 0
   End Sub

   Sub WriteResponse(ByVal lngTicks As Long)
      'Append the response message to the log and return the stream to the service
      Dim fsResponse As New FileStream(strLogFile, FileMode.Append, FileAccess.Write)
      Dim swResponse As New StreamWriter(fsResponse)
      With swResponse
         .WriteLine("<!-- SOAP Response at " + DateTime.Now.ToString + " -->")
         .Flush()
         stmTemp.Position = 0
         CopyStream(stmTemp, fsResponse)
         'Divide by 10^7 to convert from 100 ns increments to seconds
         .WriteLine("<!-- Response time = " + CStr(CDbl(lngTicks / 10000000)) + " seconds -->")
         .WriteLine("") 'Empty line
         .Flush()
      End With
      stmTemp.Position = 0
      CopyStream(stmTemp, stmSoap)
      fsResponse.Close()
   End Sub

   Public Sub CopyStream(ByVal stmSource As Stream, ByVal stmDest As Stream)
      'Copy the stream from the source to the destination
      Dim srSource As New StreamReader(stmSource)
      Dim swDest As New StreamWriter(stmDest)
      swDest.WriteLine(srSource.ReadToEnd())
      swDest.Flush()
   End Sub
End Class
